﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using Moq;
using Moq.Stub;
using Moq.Properties;
using Moq.Protected;

namespace MockTestConsole1
{
	class Program
	{
		static void Main(string[] args)
		{
			string id	= "powerumc";
			string pwd	= "aaaa";

			var mock = new Mock<ILogin>();

			// Validate
			mock.Setup( o => o.Valide(minString(3), It.IsAny<string>()))
				.Returns(LoginInputResult.MinLengthId)
				.Callback(()=>Console.WriteLine("ERROR> MinLengthId"));
			mock.Setup( o => o.Valide(maxString(10), It.IsAny<string>()))
				.Returns(LoginInputResult.MaxLengthId)
				.Callback(()=>Console.WriteLine("ERROR> MaxLengthId"));
			mock.Setup( o => o.Valide(It.IsAny<string>(), minString(3)))
				.Returns(LoginInputResult.MinLengthPassword)
				.Callback(()=>Console.WriteLine("ERROR> MinLengthPassword"));
			mock.Setup( o => o.Valide(It.IsAny<string>(), maxString(20)))
				.Returns(LoginInputResult.MaxLengthPassword)
				.Callback(()=>Console.WriteLine("ERROR> MaxLengthPassword"));

			// Login
			mock.Setup( o => o.Login(It.IsAny<string>(), It.IsAny<string>()))
				.Returns(LoginResult.NoAuthentication)
				.Callback(()=>Console.WriteLine("No Authentication"));
			mock.Setup( o => o.Login(id, pwd))
				.Returns(LoginResult.Authentication)
				.Callback(()=>Console.WriteLine("Success Login"));
			
			var obj = mock.Object;
			var loginInputResult = obj.Valide(id,pwd);

			if( loginInputResult != LoginInputResult.Success ) return;
			
			obj.Login(id,pwd);
		}

		private static string minString(int length)
		{
			return Match<string>.Create( o => o.Length <= length );
		}

		private static string maxString(int length)
		{
			return Match<string>.Create( o => o.Length >= length );
		}
	}

	public interface ILogin
	{
		LoginResult Login(string id, string password);
		LoginInputResult Valide(string id, string password);
	}

	public enum LoginResult
	{
		Authentication,
		NoAuthentication
	}

	public enum LoginInputResult
	{
		Success,
		MinLengthId,
		MaxLengthId,
		MinLengthPassword,
		MaxLengthPassword
	}

	public class LoginException : Exception { }
}
